(function($){
  // Static mixins API:
  // added as a static member to Epoxy class objects (Model & View);
  // generates a set of class attributes for mixin with other objects.
  var mixins = {
    mixin: function(extend) {
      extend = extend || {};

      for (var i in this.prototype) {
        // Skip override on pre-defined binding declarations:
        if (i === 'bindings' && extend.bindings) continue;

        // Assimilate non-constructor Epoxy prototype properties onto extended object:
        if (this.prototype.hasOwnProperty(i) && i !== 'constructor') {
          extend[i] = this.prototype[i];
        }
      }
      return extend;
    }
  };  
   
  function _super(instance, method, args) {
    return instance.__super.prototype[method].apply(instance, args);
  }
  // Base.Model
  // -----------
  var modelProps = ['computeds'];
  
  var BaseModel = Backbone.Epoxy.Model.extend({
    _super: Backbone.NestedModel,
    __super: Backbone.Epoxy.Model,
    //defaults,散列（或函数）用于为模型指定默认属性。
    //defaults : recipeDetailInfoTable,
    //在model创建后执行的方法
    //些为公用model类，定义常用的一些方法，见computeds属性，computeds可以重载覆盖公用类方法
    constructor: function(attributes, options) {
        _.extend(this, _.pick(options||{}, modelProps));
        //声名专用的显示样式字段
        var computeds = {
            //是否为最后一行
            getLast: {
                get: function() {
                    return this.get("lastRow") === null ||  typeof(this.get("lastRow")) === "undefined" || this.get("lastRow") === false ? false : true
                }
            },
            getFirst: {
                get: function() {
                    return this.get("firstRow") === null ||  typeof(this.get("firstRow")) === "undefined" || this.get("firstRow") === false ? false : true
                }
            },
            //单据创建日期
            getCreateDate: {
                deps: ["createDate"],
                get: function(createDate) {
                    if (typeof(createDate) === "object" && _.allKeys(createDate).length === 0) {
                         return null;
                    } else if(!!!createDate) {
                        return null;
                    }
                    return  commonJs.dateTimeToString(createDate.time);
                }
            },
            //单据创建日期
            getCreateDateStr: {
                deps: ["createDate"],
                get: function(createDate) {
                    if (typeof(createDate) === "object" && _.allKeys(createDate).length === 0) {
                         return null;
                    } else if(!!!createDate) {
                        return null;
                    }
                    return  commonJs.dateToString(createDate.time);
                }
            },
            //最后更新日期
            getUpdateDate: {
                deps: ["updateDate"],
                get: function(updateDate) {
                    if (typeof(updateDate) === "object" && _.allKeys(updateDate).length === 0) {
                         return null;
                    } else if(!!!updateDate) {
                        return null;
                    }
                    return commonJs.dateTimeToString(updateDate.time);
                }
            }
        }
        
        //检查对像是否有传入, computeds属有，有则进行对比，外部同名方法，覆盖掉内部方法
        var computedsCheck = function(innerObj, key) {
           for (var item in innerObj) {
               if (item === key) {
                   delete(innerObj[key]);
                   break;
               }
           }
        }        
        var selfComputeds = this["computeds"] || {};
        for (var item in selfComputeds) {
            computedsCheck(computeds, item);
        }
        for(var key in computeds) {
            selfComputeds[key] = computeds[key];
        }
        this["computeds"] = selfComputeds || {};
        
        //执行父类重购方法
        _super(this, 'constructor', arguments);
        this.set("styleDisplay", "-webkit-box !important");
        this.set("firstRow", false); //是否第一行  collection渲染使用
        this.set("lastRow", false);  //是否最后一行  collection渲染使用
    },
    clearData: function() {
        var self = this;
        for (var key in self.attributes) {
            self.attributes[key] = "";
        }
    }
  }, mixins);
  
  
  /**
   *  BaseCollection
   */
   var __set = function(instance, method, models, options) {
      return instance.__super.prototype[method](models, options);
  }
  
    // Splices `insert` into `array` at index `at`.
  var splice = function(array, insert, at) {
    at = Math.min(Math.max(at, 0), array.length);
    var tail = Array(array.length - at);
    var length = insert.length;
    var i;
    for (i = 0; i < tail.length; i++) tail[i] = array[i + at];
    for (i = 0; i < length; i++) array[i + at] = insert[i];
    for (i = 0; i < tail.length; i++) array[i + length + at] = tail[i];
  };

  // Default options for `Collection#set`.
  var setOptions = {add: true, remove: true, merge: true};
  var addOptions = {add: true, remove: false};
  
  var BaseCollection = Backbone.Collection.extend({
      __super: Backbone.Collection,
      // Update a collection by `set`-ing a new list of models, adding new ones,
    // removing models that are no longer present, and merging models that
    // already exist in the collection, as necessary. Similar to **Model#set**,
    // the core operation for updating the data contained by the collection.
    set: function(models, options) {
      if (models == null) return;

      options = _.extend({}, setOptions, options);
      if (options.parse && !this._isModel(models)) {
        models = this.parse(models, options) || [];
      }

      var singular = !_.isArray(models);
      models = singular ? [models] : models.slice();

      var at = options.at;
      if (at != null) at = +at;
      if (at > this.length) at = this.length;
      if (at < 0) at += this.length + 1;

      var set = [];
      var toAdd = [];
      var toMerge = [];
      var toRemove = [];
      var modelMap = {};

      var add = options.add;
      var merge = options.merge;
      var remove = options.remove;

      var sort = false;
      var sortable = this.comparator && at == null && options.sort !== false;
      var sortAttr = _.isString(this.comparator) ? this.comparator : null;

      // Turn bare objects into model references, and prevent invalid models
      // from being added.
      var model, i;
      for (i = 0; i < models.length; i++) {
        model = models[i];

        // If a duplicate is found, prevent it from being added and
        // optionally merge it into the existing model.
        var existing = this.get(model);
        if (existing) {
          if (merge && model !== existing) {
            var attrs = this._isModel(model) ? model.attributes : model;
            if (options.parse) attrs = existing.parse(attrs, options);
            existing.set(attrs, options);
            toMerge.push(existing);
            if (sortable && !sort) sort = existing.hasChanged(sortAttr);
          }
          if (!modelMap[existing.cid]) {
            modelMap[existing.cid] = true;
            set.push(existing);
          }
          models[i] = existing;

        // If this is a new, valid model, push it to the `toAdd` list.
        } else if (add) {
          model = models[i] = this._prepareModel(model, options);
          if (model) {
            toAdd.push(model);
            this._addReference(model, options);
            modelMap[model.cid] = true;
            set.push(model);
          }
        }
      }

      // Remove stale models.
      if (remove) {
        for (i = 0; i < this.length; i++) {
          model = this.models[i];
          if (!modelMap[model.cid]) toRemove.push(model);
        }
        if (toRemove.length) this._removeModels(toRemove, options);
      }

      // See if sorting is needed, update `length` and splice in new models.
      var orderChanged = false;
      var replace = !sortable && add && remove;
      if (set.length && replace) {
        orderChanged = this.length !== set.length || _.some(this.models, function(m, index) {
          return m !== set[index];
        });
        this.models.length = 0;
        splice(this.models, set, 0);
        this.length = this.models.length;
      } else if (toAdd.length) {
        if (sortable) sort = true;
        splice(this.models, toAdd, at == null ? this.length : at);
        this.length = this.models.length;
      }

      // Silently sort the collection if appropriate.
      if (sort) this.sort({silent: true});

      // Unless silenced, it's time to fire all appropriate add/sort/update events.
      if (!options.silent) {
        for (i = 0; i < toAdd.length; i++) {
          if (at != null) options.index = at + i;
          model = toAdd[i];
          model.trigger('add', model, this, options);
        }
        if (sort || orderChanged) this.trigger('sort', this, options);
        if (toAdd.length || toRemove.length || toMerge.length) {
          options.changes = {
            added: toAdd,
            removed: toRemove,
            merged: toMerge
          };
          this.trigger('update', this, options);
        }
      }
      //add by lizehn 2018-09-16
      for(var i = 0; i < this.length; i++) {
          this.models[i].set("firstRow", i === 0);
          this.models[i].set("lastRow", i + 1 === this.length);
      }
      // Return the added (or merged) model (or models).
      return singular ? models[0] : models;
    }
      //构选函数
      // constructor: function(options) {
        // _.extend(this, _.pick(options||{}));
        // //执行父类重购方法
        // _super(this, 'constructor', arguments);
      // }
  });
  
  _.extend(BaseCollection.prototype, Backbone.Events, {
      setPublicAttribute: function(model) {
          var index = this.indexOf(model);
          //设置数据是否为第一行
          if (index === 0) {
              var _models = this.where({firstRow: true});
              for (var i = 0; i < _models.length; i++) {
                  _models[i].set("firstRow", false);
              }
              model.set("firstRow", true);
          }
          //设置数据为最后一行
          if (index + 1 === this.length) {
              var _models = this.where({lastRow: true});
              for (var i = 0; i < _models.length; i++) {
                  _models[i].set("lastRow", false);
              }
              model.set("lastRow", true);
          }
      }
  });
  
  /**
   * BaseService
   */
  var BaseService = Backbone.Service.extend({
      __super: Backbone.Service
  });
  
  _.extend(BaseService.prototype, Backbone.Events, {});
  
  Backbone.BaseModel = BaseModel;
  Backbone.BaseCollection = BaseCollection;
  Backbone.BaseService = BaseService;
})($);

var CommonBase = {
    Model: Backbone.BaseModel,
    Collection: Backbone.BaseCollection,
    Service: Backbone.BaseService
}

var commonService = new CommonBase.Service({    
    pretreatment : function(data, option) {
        return data;
    },
    dosuccess : function(data, option) {
        return data;
    },
    doerror : function(e, err, option) {
        return err;
    },
    validate : function(data, option) {
        return 0;
    },
    ajaxCall : function(data, option) {
        var self = this;
        if (!!!option.url) {
            appcan.window.alert("", "Service请求未传入URL参数，提交AJAX请求失败");
            return;
        }
        var param = JSON.stringify(data);
        if (!!option.showLoading) {
            commonJs.showLoading();
        }
        
        commonJs.requestAjax(option.url, param, function(res) {
            option.success(self.dosuccess(res, option));
            if (!!option.showLoading) {
                commonJs.closeLoading();
            }
        }, function (e, err) {
            option.error(self.doerror(e, err, option));
            if (!!option.showLoading) {
                commonJs.closeLoading();
            }
        });
        
    }
});