var DoctorInfoModel = CommonBase.Model.extend({
    //defaults,散列（或函数）用于为模型指定默认属性。
    defaults : doctorInfoTable,
    //在model创建后执行的方法
    initialize : function() {
        //console.log("数据初始化");
        return;
    },
    // 默认情况下validate在save之前调用，也可以在set之前调用。 如果validate返回一个错误， save不会继续，该模型的属性将不被修改。
    validate : function(attrs, options) {
        //console.log(attrs, options);
    },
    //可以理解为对应model的实时计算，比如这里的model里面的username发生了变化，
    //那么对应的<span data-bind="text:add"></span>也会发生变化
    computeds : {
        //医师类别 
        getDoctorType : {
            //这里是参数的依赖，也可以使用this.get("username")
            deps : ["doctorType"],
            get : function(doctorType) {
                var codelist = commonJs.getCodelistData(window.codeList.doctorType, doctorType);
                return codelist.text || doctorType;
            },
        },
        //性别
        getSex : {
            //这里是参数的依赖，也可以使用this.get("username")
            deps : ["sex"],
            get : function(sex) {
                var codelist = commonJs.getCodelistData(window.codeList.genderType, sex);
                return codelist.text || sex;
            }
        },
        //学历
        getEducation : {
            //这里是参数的依赖，也可以使用this.get("username")
            deps : ["education"],
            get : function(education) {
                var codelist = commonJs.getCodelistData(window.codeList.education, education);
                return codelist.text || education;
            }
        },
        //头像
        getHeadPicture : {
            //这里是参数的依赖，也可以使用this.get("username")
            deps : ["headPicture"],
            get : function(headPicture) {
            	if(!!headPicture && headPicture.indexOf("blob:file") != '-1'){
            		return headPicture;
            	} else if(!!headPicture && headPicture.indexOf("data:image") != '-1'){
                    return headPicture;
                }
                return !!headPicture ? window.PicURL + headPicture : "";
            }
        },
        //身证份正面
        getCardIdPicture1 : {
            //这里是参数的依赖，也可以使用this.get("username")
            deps : ["cardIdPicture1"],
            get : function(cardIdPicture1) {
            	if(!!cardIdPicture1 && cardIdPicture1.indexOf("../") != '-1'){
            		return cardIdPicture1;
            	}
                return window.PicURL + cardIdPicture1;
            }
        },
        //身份证反面
        getCardIdPicture2 : {
            //这里是参数的依赖，也可以使用this.get("username")
            deps : ["cardIdPicture2"],
            get : function(cardIdPicture2) {
            	if(!!cardIdPicture2 && cardIdPicture2.indexOf("../") != '-1'){
            		return cardIdPicture2;
            	}
                return window.PicURL + cardIdPicture2;
            }
        },
        //医师执业证
        getBusinessPicture1 : {
            //这里是参数的依赖，也可以使用this.get("username")
            deps : ["businessPicture1"],
            get : function(businessPicture1) {
            	if(!!businessPicture1 && businessPicture1.indexOf("../") != '-1'){
            		return businessPicture1;
            	}
                return window.PicURL + businessPicture1;
            }
        },
        //医师执业证详情页
        getBusinessPicture2 : {
            //这里是参数的依赖，也可以使用this.get("username")
            deps : ["businessPicture2"],
            get : function(businessPicture2) {
            	if(!!businessPicture2 && businessPicture2.indexOf("../") != '-1'){
            		return businessPicture2;
            	}
                return window.PicURL + businessPicture2;
            }
        },
        //专业技师执业证
        getTitlePicture1 : {
            //这里是参数的依赖，也可以使用this.get("username")
            deps : ["titlePicture1"],
            get : function(titlePicture1) {
            	if(!!titlePicture1 && titlePicture1.indexOf("../") != '-1'){
            		return titlePicture1;
            	}
                return window.PicURL + titlePicture1;
            }
        },
        //专业技师执业证详情页
        getTitlePicture2 : {
            //这里是参数的依赖，也可以使用this.get("username")
            deps : ["titlePicture2"],
            get : function(titlePicture2) {
            	if(!!titlePicture2 && titlePicture2.indexOf("../") != '-1'){
            		return titlePicture2;
            	}
                return window.PicURL + titlePicture2;
            }
        },
        //医生是否登陆,有ID值则表示已登陆
        getIsLogin: {
            deps: ["id"],
            get: function(id) {
                return !!id;
            }
        },
        getCertFlag: {
            deps: ["certStatus"],
            get: function(certStatus) {
                if (certStatus === "YRZ") {
                    return true;
                } else {
                    return false;
                }
            }
        },
        getCertFlag1: {
            deps: ["certStatus"],
            get: function(certStatus) {
                if (certStatus === "WRZ") {
                    return true;
                } else {
                    return false;
                }
            }
        },
        getPraiseRate: {
            deps: ["praiseRate"],
            get: function(praiseRate) {
                if (!!this.get("id")) {
                    return !!praiseRate ? Number(praiseRate) * 100 + "%" : "0%";
                }
                return "";
            }
        }
    },
    parse : function(data) {
        return data;
    },
    sync : function(method, model, options) {
        switch (method) {
        case "create":
            break;
        case "update":
            break;
        case "patch":
            break;
        case "delete":
            break;
        case "read":
            break;
        default:
            break;
        }
    }
});